clear
clc
%define Weight
Wn = 0.001;
Wu = 10*tf([1 10],[1 1000]);
Wdist = tf(0.25,[1 0.25],'outputname','f2');
Wp = 80*tf(0.1,[1 0.1]);
%define System Parameters
m1 = 1;
m2 = 2;
k2 = 1;
b1 = 0.05;
b2 = 0.05;
% this System Parameter is Real Uncertainty
k1 = ureal('k1',2,'Range',[1.2 2.8]);
%define Nominal system
a1c = [0 0 -1/m1  1/m2]'*k1;
a2c = [0 0  1/m1 -1/m2]'*k1 + [0 0 0 -k2/m2]';
a3c = [1 0 -b1/m1 b1/m2]';
a4c = [0 1 b1/m1 -(b1+b2)/m2]';
A  = [a1c a2c a3c a4c];
plant = ss(A,[0 0;0 0;1/m1 0;0 1/m2],[0 1 0 0],[0 0]);
plant.StateName = {'x1';'x2';'x1dot';'x2dot'};
plant.OutputName = {'x2'};
%define Uncertainty system
Wunmod = 2.6*tf([1 0],[1 40]);
Delta = ultidyn('Delta',[1 1]);
plant = plant * append(1+Delta*Wunmod,1);
plant.InputName = {'f1 (control)','f2 (disturbance)'};
%Create Standard configuration
systemnames = 'plant Wu Wp Wn Wdist';
inputvar = '[dist; noise; f1]';
outputvar = '[Wp; Wu; plant+Wn]';
input_to_Wdist = '[dist]';
input_to_plant = '[f1; Wdist]';
input_to_Wu = '[f1]';
input_to_Wp = '[plant]';
input_to_Wn = '[noise]';
cleanupsysic = 'yes';
IC = sysic;
opt = dksynOptions('NumberofAutoIterations',4,'DisplayWhileAutoIter','on');
[Kc,gc,mu_c,infoc] = dksyn(IC,1,1,opt);
K1=zpk(infoc{1}.K);
K4=zpk(infoc{4}.K);
figure(3)
bodemag(K1,K4,'k-.')
legend('K_1', 'K_4')

figure(1)
clf
semilogx(infoc{1}.MussvBnds(:,1),'b',infoc{2}.MussvBnds(:,1),'k-.', ...
infoc{3}.MussvBnds(:,1),'--g',infoc{4}.MussvBnds(:,1),':r')
legend('\mu_1', '\mu_2', '\mu_3', '\mu_4')
xlabel('Frequency (rad/sec)')
title('Structured singular values for complex uncertainty')


figure(2)
clf
% Mixed uncertainty
opt = dksynOptions('MixedMU','on','AutoScalingOrder',[5 6], ...
    'NumberofAutoIterations',4,'DisplayWhileAutoIter','on');
[Km,gm,mu_m,infom] = dksyn(IC,1,1,opt);
semilogx(infom{1}.MussvBnds(:,1),'b',infom{4}.MussvBnds(:,1),'-.k')
legend('\mu_1', '\mu_4')
xlabel('Frequency (rad/sec)')
title('Structured singular values for mixed uncertainty')
